%{
#include <malloc.h>
#include <string.h>
#include "parser.h"
#include "structs.h"
#define MAXLXSTR 65536

extern struct s_object *objtable;

void yyerror(char *);
%}

			char lxstrbuf[MAXLXSTR];
			char *lxstrptr;
			int lxstrlen;

%x strmode

%%

^"#".*\n		;
[a-zA-Z][a-zA-Z0-9]*	{
				yylval.obj = objtable;
				while(yylval.obj
				&& strcmp(yylval.obj->name, yytext))
					yylval.obj = yylval.obj->tblnext;
				if(!yylval.obj) {
					yylval.obj = (struct s_object *)
					malloc(sizeof(struct s_object));
					yylval.obj->tblnext = objtable;
					objtable = yylval.obj;
					yylval.obj->name = strdup(yytext);
					yylval.obj->state =
					  OBJSTATE_UNDEF;
					/*yylval.obj->willbeborn = 0;*/
					yylval.obj->newlyborn = 0;
				}
				return TOK_SYMBOL;
			}
[0-9]+			{
				yylval.intval = atoi(yytext);
				return TOK_INTEGER;
			}
"<-"			return TOK_LEFTARROW;
"*"			return TOK_STAR;
"&"			return TOK_AMPERSAND;
"~"			return TOK_TILDE;
"("			return TOK_LEFT;
")"			return TOK_RIGHT;
\"			{
				BEGIN(strmode);
				lxstrlen = 0;
				lxstrptr = lxstrbuf;
			}
<strmode>\"		{
				BEGIN(INITIAL);
				*lxstrptr = 0;
				yylval.str = strdup(lxstrbuf);
				return TOK_STRING;
			}
<strmode>\\n		{
				if(lxstrlen >= MAXLXSTR) {
					yyerror("string constant too long");
				}
				*lxstrptr++ = '\n';
				lxstrlen++;
			}
<strmode>\\\\		{
				if(lxstrlen >= MAXLXSTR) {
					yyerror("string constant too long");
				}
				*lxstrptr++ = '\\';
				lxstrlen++;
			}
<strmode>\\\"		{
				if(lxstrlen >= MAXLXSTR) {
					yyerror("string constant too long");
				}
				*lxstrptr++ = '"';
				lxstrlen++;
			}
<strmode>\\.		{
				if(lxstrlen+1 >= MAXLXSTR) {
					yyerror("string constant too long");
				}
				*lxstrptr++ = '\\';
				*lxstrptr++ = yytext[1];
				lxstrlen+=2;
			}
<strmode>[^"\\]+	{
				if(lxstrlen+yyleng >= MAXLXSTR) {
					yyerror("string constant too long");
				}
				strcpy(lxstrptr, yytext);
				lxstrptr = &lxstrptr[yyleng];
				lxstrlen += yyleng;
			}
<strmode><<EOF>>	{
				yyerror("eof in string constant");
			}
[ \t\n]*		;
.			{
				fprintf(stderr, "unexpected: %x '%c'\n",
					yytext[0], yytext[0]);
			}
