%{

#include <string.h>
#include <malloc.h>

#include "structs.h"

extern struct s_diagram *diagram;

struct s_object *objtable = NULL;

void yyerror(char *);
int yylex();
%}

%union {
	char *str;
	int intval;
	struct s_diagram *dia;
	struct s_action *act;
	struct s_message *msg;
	struct s_object *obj;
	struct s_text *txt;
}

%type <dia> diagram
%type <act> action actions
%type <msg> message callee deletemsg
%type <obj> objdefs objdef
%type <txt> textdef textdefs
%type <str> return

%token <str> TOK_STRING
%token <obj> TOK_SYMBOL
%token <intval> TOK_INTEGER
%token TOK_LEFTARROW
%token TOK_STAR
%token TOK_LEFT
%token TOK_RIGHT
%token TOK_AMPERSAND
%token TOK_TILDE

%expect 1

%%

diagram:	  objdefs actions textdefs
			{
				$$ = (struct s_diagram *)
				malloc(sizeof(struct s_diagram));
				$$->objects = $1;
				$$->seq = $2;
				$$->texts = $3;
				diagram = $$;
			}
		;

objdefs:	  objdefs objdef
			{
				$$ = $2;
				$$->defnext = $1;
			}
		| /* empty */
			{ $$ = NULL; }
		;
objdef:		  TOK_LEFT TOK_SYMBOL TOK_STRING TOK_INTEGER TOK_INTEGER TOK_RIGHT
			{
				$$ = $2;
				$$->label = $3;
				$$->xpos = $4;
				$$->width = $5;
				$$->state = OBJSTATE_INACTIVE;
			}
		| TOK_LEFT TOK_SYMBOL TOK_STRING TOK_INTEGER TOK_RIGHT
			{
				$$ = $2;
				$$->label = $3;
				$$->xpos = $4;
				$$->width = 0;
				$$->state = OBJSTATE_INACTIVE;
			}
		;
actions:	  action actions /* right-handed recursion */
			{
				$$ = $1;
				$$->next = $2;
			}
		| /* empty */
			{ $$ = NULL; }
		;
action:		  message actions return
			{
				$$ = (struct s_action *)
				malloc(sizeof(struct s_action));
				$$->msg = $1;
				$$->whathappen = $2;
				$$->retval = $3;
				$$->deletion = 0;
				if($1->creation) {
					$$->whathappen = (struct s_action *)
					malloc(sizeof(struct s_action));
					$$->whathappen->msg = NULL;
					$$->whathappen->whathappen = NULL;
					$$->whathappen->next =
					(struct s_action *)
					malloc(sizeof(struct s_action));
					$$->whathappen->next->msg = NULL;
					$$->whathappen->next->whathappen = NULL;
					$$->whathappen->next->next = $2;
				}
			}
		| TOK_TILDE deletemsg
			{
				$$ = (struct s_action *)
				malloc(sizeof(struct s_action));
				$$->msg = $2;
				$$->whathappen = NULL;
				$$->retval = NULL;
				$$->deletion = 1;
			}
		| TOK_AMPERSAND
		 	{
				$$ = (struct s_action *)
				malloc(sizeof(struct s_action));
				$$->msg = NULL;
				$$->whathappen = NULL;
			}
		;
message:	  callee TOK_STRING
			{
				$$ = $1;
				$$->msg = $2;
			}
		| callee
		;

deletemsg:	  TOK_SYMBOL TOK_STRING
			{
				$$ = (struct s_message *)
				malloc(sizeof(struct s_message));
				$$->obj = $1;
				$$->msg = $2;
				$$->creation = 0;
			}
		| TOK_SYMBOL
			{
				$$ = (struct s_message *)
				malloc(sizeof(struct s_message));
				$$->obj = $1;
				$$->msg = "";
				$$->creation = 0;
			}
		;

callee:		  TOK_SYMBOL
			{
				$$ = (struct s_message *)
				malloc(sizeof(struct s_message));
				$$->obj = $1;
				$$->msg = "";
				$$->creation = 0;
			}
		| TOK_STAR TOK_SYMBOL
			{
				$$ = (struct s_message *)
				malloc(sizeof(struct s_message));
				$$->obj = $2;
				$$->msg = "";
				$$->creation = 1;
			}
		;
return:		  TOK_LEFTARROW TOK_STRING
			{ $$ = $2; }
		| TOK_LEFTARROW
			{ $$ = ""; }
		;

textdefs:	  /* empty */
			{ $$ = NULL; }
		| textdefs textdef
			{
				$$ = $2;
				$$->next = $1;
			}
		;

textdef:	  TOK_LEFT TOK_STRING TOK_INTEGER TOK_INTEGER TOK_RIGHT
			{
				$$ = (struct s_text *)
				malloc(sizeof(struct s_text));
				$$->text = $2;
				$$->xpos = $3;
				$$->ypos = $4;
			}
		;

%%

void yyerror(char *s) {
	fprintf(stderr, "%s\n", s);
	exit(1);
}

